require "panoramix/plugin/wget"
require "panoramix/plugin/s3"
require "panoramix/plugin/git"
require "panoramix/plugin/docker_image"
require "panoramix/mpi"

def external(dep, &block)
  # Provided value can be either array or hash, so we need to make sure the final value is an array
  value = [dep.values[0]].flatten

  # Select the hash with params
  params = value.select { |d| d.is_a? Hash }.first

  # Handle with MPI
  handled_dep = Panoramix::MPI.handle_external(dep.keys[0], params)

  raise "Only a single external dependency supported (received: #{dep.keys})" if handled_dep.length != 1

  # Select provided dependencies
  prerequisites = value.select { |d| ! d.is_a? Hash }

  case handled_dep.values[0].keys[0]
  when :s3
    s3(handled_dep, prerequisites, block)
  when :wget
    wget(handled_dep, prerequisites, block)
  when :git
    git(handled_dep, prerequisites, block)
  when :dockerimage
    external_docker_image(handled_dep, block)
  when :docker
    puts "Warning using obsolete external dependency key
      \t => external <image_name> {#{"docker".bold}: <docker_image_name> }.\n\tchange to:
      \t => external <image_name> {#{"dockerimage".bold}: <docker_image_name> }.".red
    external_docker_image(handled_dep, block)
  else
    raise "Unknown external protocol '#{dep.values[0].keys[0]}'"
  end
end
