require 'panoramix/locale'
require 'open3'
require 'colorize'

module Panoramix
	module Plugin

		class ValidationError < StandardError; end

		class Base

			# Return current timestamp
			def timestamp
				raise "Not implemented"
			end

			# When this instance needs to be executed
			def needed? timestamps
				raise "Not implemented"
			end

			# Has this instance already been created
			def created?
				raise "Not implemented"
			end

			# Default action for this task
			def run_default
				raise "Not implemented"
			end

			def shell(cmd, silent=false, env=Hash.new)
	      puts "Running #{cmd}  ENV=#{env}".magenta if ENV["VERBOSE"]

	      @err = ""
	      @out = ""
	      Open3.popen3(env, cmd) do |stdin, stdout, stderr, wait_thr|
	        stdin.close
	        stdout.each_line do |line|
	          puts line unless silent
	          @out << "#{line}"
	        end
	        stderr.each_line do |line|
	          puts line.red unless silent
	          @err << "#{line}"
	        end
	        @exit_status = wait_thr.value
	      end

	      if ENV["VERBOSE"] && !@out.empty?
	        puts "\tout => \n\t\t#{@out.gsub("\n","\n\t\t")}".green
	      end
	      if ENV["VERBOSE"] && !@err.empty?
	        if @exit_status.success?
	          puts "\terr => \n\t\t#{@err.gsub("\n","\n\t\t")}".yellow
	        else
	          puts "\terr => \n\t\t#{@err.gsub("\n","\n\t\t")}".red
	        end
	      end
	      puts "\texit_status => #{@exit_status}\n".blue if ENV["VERBOSE"]

	      raise @err unless @exit_status.success?

	      return {
	        :out => @out,
	        :err => @err,
	        :exit_status => @exit_status
	      }
	    end
	    
		end
	end
end
