require "panoramix/plugin/docker_image_base"

module Panoramix
  module Plugin

    class DockerBuild < DockerImageBase

      attr_reader :env
      attr_reader :context
      attr_reader :dockerfile

      def initialize(dst, dockerfile, context, host)
        super(dst, host)
        @dockerfile = dockerfile 
        @context = context
        @env = Hash.new
        @env["DOCKER_HOST"] = "tcp://#{host}" if host
      end

      # When this instance needs to be executed
      def needed? timestamps
        return false if ENV['NO_BUILD'] && ENV['NO_BUILD'].split(":").any? { |image| image == @dst }
        return true
      end

      # Default action for this task
      def run_default
        shell("docker build -t #{dst} -f #{@dockerfile} #{@context}", false, @env)
      end

      def ps
        super ("Built image")
      end

    end

  end
end

def docker_image(args, block=nil)
  puts "Warning using obsolete docker_image
    \t => #{"docker_image".bold} <image_name> => {dockerfile: <path_dockerfile>}.\n\tchange to:
    \t => #{"docker_build".bold} <image_name> => {dockerfile: <path_dockerfile>}".red
  docker_build(args, block)
end

def docker_build(args, &block)
  # Get task name
  name = args.keys[0]

  # Get configuration hash
  config = args[name].select { |dep| dep.is_a? Hash }.first

  # Get dockerfile if given
  dockerfile = config.fetch(:dockerfile, nil)

  # Raise error if :dockerfile value is nil
  unless dockerfile
    message = I18n.t('errors.docker_build.no_dockerfile_provided')
    raise message
  end

  # Get context if given
  context = config.fetch(:context, File.dirname(dockerfile))

  # Select the host
  host = config.fetch(:host, nil)

  # Select provided dependencies
  prerequisites = args[name].select { |d| ! d.is_a? Hash }

  # Merge prerequisites with the Dockerfile file_task
  prerequisites = prerequisites.push(dockerfile) unless dockerfile.nil?
  prerequisites.flatten!

  instance = Panoramix::Plugin::DockerBuild.new(name, dockerfile, context, host)

  descriptions = I18n.t('docker_build')
  descriptions = Hash.new if descriptions.class != Hash

  Panoramix.define_tasks(name,descriptions, instance, prerequisites, block)
end
