require "panoramix/plugin/docker_image_base"

module Panoramix
	module Plugin

		class DockerImage < DockerImageBase

			attr_reader :src
			attr_reader :env

			def initialize(dst, src, host)
				super(dst, host)
				@src = src
				@env = Hash.new
				@env["DOCKER_HOST"] = "tcp://#{host}" if host
			end

			# When this instance needs to be executed
			def needed? timestamps
				true
			end

			# Default action for this task
			def run_default
				shell("docker pull #{@src}", false, @env) unless ENV["NO_PULL"]

				# Tag image with required tag
				shell("docker tag #{@src} #{@dst}", false, @env)

				# Remove origin image
				# shell("docker rmi #{@src}", false, @env)
				# It fails when the base image is in use.
			end

			def ps
				super ("Pulled image")
			end

		end

	end
end


def external_docker_image(args, block)
	name = args.keys[0]
	src = args[name][:dockerimage] || args[name][:docker]

	host = args[name][:host] || nil

	prerequisites = []

	instance = Panoramix::Plugin::DockerImage.new(name, src, host)

	descriptions = I18n.t('docker_image')
	descriptions = Hash.new if descriptions.class != Hash

	Panoramix.define_tasks(name, descriptions, instance, prerequisites, block)
end
