require "time"
require "panoramix/plugin/base"

module Panoramix
  module Plugin

    class DockerImageBase < Base

      attr_reader :dst
      attr_reader :tag
      attr_reader :env

      def initialize(dst, host)
        @dst = dst
        @tag = "latest"
        @env = Hash.new
        @env["DOCKER_HOST"] = "tcp://#{host}" if host
      end

      # Return timestamp of the image
      def timestamp
        info =  created?
        if info
          # Get image timestamp
          id = info.split(" ")[0]
          time = shell("docker inspect -f {{.Created}} #{id}", true, @env)[:out]
          return Time.parse(time)
        else
          return Time.at 0
        end
      end

      # Has this image already been created
      def created?
        return @created if @created
        
        info = shell("docker images", true, @env)[:out]

        images_list = info.split("\n")
        images_list.shift
        image_info = images_list.select {|img| img.match(/#{@dst}\s+#{@tag}/) }

        @created = info = info.empty? ? nil: image_info[0]
      end

      # Action clobber for this task
      def clobber
        info = created?
        if info
          id = info.split(" ")[0]
          shell("docker rmi -f #{id}", false, @env)
        end 
      end

      def ps(message)
        puts "#{message}: #{@dst}"
        info = created?
        if info
          puts "Timestamp: #{timestamp}"
          puts info
        else
          puts "Timestamp: Not created"
        end
        puts
      end

    end
  end
end
