require 'time'
require 'panoramix/plugin/base'

module Panoramix
  module Plugin

    class EnvironmentError < StandardError; end

    class Environment < Base

      attr_reader :dst
      attr_reader :src

      def initialize(dst, src)
        @dst = dst
        @src = src
      end

      # Always return an old time
      def timestamp
        return Time.at(0)
      end

      # Always true
      def needed? timestamps
        true
      end

      # Deafult task
      def run_default
        @src.each do |var|
          raise "Environment variable #{var} not defined" unless ENV[var]
        end
      end

      def vars
        puts @dst.bold
        @src.each do |var|
          puts "#{var}=#{ENV[var]}"
        end
        puts ""
      end
    end

  end
end

def env(args, &block)
  name = ""

  deps = []
  # case env "URL"
  if (args.class == String) 
    name = args
    vars = [name]
  else  # case env "some_var" => [{:vars => [......]}]
    name = args.keys[0]
    
    deps = args[name]
    env_options = deps.select { |d| d.is_a? Hash }
    # Get variables array
    vars = env_options.empty? ? nil : env_options.first[:vars]
  end

  descriptions = I18n.t('env')
  descriptions = Hash.new if descriptions.class != Hash

  # Merge deps
  prerequisites = deps.select { |d| ! d.is_a? Hash }

  instance = Panoramix::Plugin::Environment.new(name, vars)

  Panoramix.define_tasks(name, descriptions, instance, prerequisites, block)
end