require "time"
require "panoramix/plugin/base"

module Panoramix
	module Plugin
		class Git < Base

			attr_reader :dst
			attr_reader :src
			attr_reader :tag

			def initialize(dst, src, tag)
				# Get git project name
			  repo_name=src.split("/").last.gsub(".git", "")

			  # Base path under which project will be downloaded
			  base_path=dst.split(repo_name)[0..-1].join(repo_name)

			  @dst=File.join(base_path, repo_name)
				@src = src
				@tag = tag
			end

			# Return current timestamp
			def timestamp
				return Time.at 0 unless created?
		    git_time = shell("git -C #{@dst} log -1 --format=\"%cd\"", true)[:out]
		    #TODO check exit code
		    Time.parse(git_time)
			end

			# When this instance needs to be executed
			def needed? timestamps
				return true if !created?
				# Get remote branches
				remote = shell("git ls-remote #{src}", true)[:out].split("\n")

				# Is @tag a branch?
				is_branch = remote.select { |r| r.include?("refs/heads/#{tag}") }

				if is_branch.any?
					# Compare local refs with remote ref
					remote_ref = is_branch.first.gsub /\t/, ' '
					local_refs = shell("git -C #{@dst} show-ref", true)[:out].split("\n")

					return !local_refs.include?(remote_ref)
				end

				#TODO: To be implemented for tags and commits

				return true
			end

			# Has this instance already been created
			def created?
				File.directory?(@dst)
			end

			# Action clean fot this task
			def clean
				shell "rm -rf #{@dst}"
			end

			# Default action for this task
			def run_default
				if created?
		      shell "git -C #{@dst} pull"
		    # File does not exists, so clone the full repository
		    else
		      shell "git clone #{@src} #{@dst}"
		    end

		    # Switch to tag
		    shell "git -C #{@dst} checkout #{@tag}"
			end
			
		end
	end
end

def git(args, prerequisites, block=nil)
	# Name of the task
	name = args.keys[0]
	# Get :git value from arguments
	src = args.values[0][:git]
	# Get :tag value from arguments
	tag = args.values[0][:tag]

	# Generate a new instance using the provided arguments
	instance = Panoramix::Plugin::Git.new(name, src, tag)

	descriptions = I18n.t('git')
	descriptions = Hash.new if descriptions.class != Hash

	# Define git instance tasks
	Panoramix.define_tasks(name, descriptions, instance, prerequisites, block)
end
