require "time"
require "panoramix/plugin/base"

module Panoramix
	module Plugin
		class S3 < Base

			attr_reader :dst
			attr_reader :src
			attr_reader :flags

			def initialize(dst, src, flags)
				@dst = dst
				@src = src
				@flags = flags
			end

			# Return current timestamp
			def timestamp
				return Time.at 0 unless created?
				remote_time = shell("aws s3 ls --summarize #{@src} | head -n1 | awk 'END {print $1, $2}'", true)[:out]
				while remote_time == " \n"
					sleep 1
					remote_time = shell("aws s3 ls --summarize #{@src} | head -n1 | awk 'END {print $1, $2}'", true)[:out]
				end
			  remote_time = Time.parse(remote_time)
			end

			# When this instance needs to be executed
			def needed? timestamps
				return true if !created?
				remote_size = shell("aws s3 ls --summarize #{@src} | head -n1 | awk 'END {print $3}'", true)[:out]
				File.mtime(@dst) < timestamp || File.size(@dst) != remote_size.to_i
			end

			# Has this instance already been created
			def created?
				File.exists?(@dst)
			end

			# Action clean fot this task
			def clean
				shell "rm -f #{@dst}"
			end

			# Default action for this task
			def run_default
				shell "aws s3 cp #{@flags} #{@src} #{@dst}"
			end

		end
	end
end


def s3(args, prerequisites, block=nil)
	# Task name
	name = args.keys[0]
	# Artifacts url
	src = args.values[0][:s3]
	# Provided flags
	flags = args.values[0][:flags]

	instance=Panoramix::Plugin::S3.new(name, src, flags)

	descriptions = I18n.t('s3')
	descriptions = Hash.new if descriptions.class != Hash

	Panoramix.define_tasks(name, descriptions, instance, prerequisites, block)
end
