require "time"
require "panoramix/plugin/base"

module Panoramix
	module Plugin
		class Wget < Base

			attr_reader :dst
			attr_reader :src
			attr_reader :flags

			def initialize(dst, src, flags)
				@dst = dst
				@src = src
				@flags = flags
			end

			# Return current timestamp
			def timestamp
				return Time.at 0 unless created?
				remote_time = shell("wget #{@flags} --spider -S #{@src} 2>&1 | grep Last-Modified", true)[:out]
			    remote_time = remote_time.split("Last-Modified: ").last
			    remote_time = Time.parse(remote_time)
			end

			# When this instance needs to be executed
			def needed? timestamps
				return true if !created?
				File.mtime(@dst) < timestamp
			end

			# Has this instance already been created
			def created?
				File.exists?(@dst)
			end

			# Action clean fot this task
			def clean
				shell "rm -f #{@dst}"
			end

			# Default action for this task
			def run_default
				shell "wget -nv #{@flags} #{@src} -O #{@dst}"
			end

		end
	end
end

def wget(args, prerequisites, block=nil)
	# Task name
	name = args.keys[0]
	# Artifacts url
	src = args.values[0][:wget]
	# Provided flags
	flags = args.values[0][:flags]

	instance=Panoramix::Plugin::Wget.new(name, src, flags)

	descriptions = I18n.t('wget')
	descriptions = Hash.new if descriptions.class != Hash

	Panoramix.define_tasks(name, descriptions, instance, prerequisites, block)
end
